/**************************************************************************
**
** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
** Contact: http://www.qt-project.org/legal
**
** This file is part of the Qt Installer Framework.
**
** $QT_BEGIN_LICENSE:LGPL$
** Commercial License Usage
** Licensees holding valid commercial Qt licenses may use this file in
** accordance with the commercial license agreement provided with the
** Software or, alternatively, in accordance with the terms contained in
** a written agreement between you and Digia.  For licensing terms and
** conditions see http://qt.digia.com/licensing.  For further information
** use the contact form at http://qt.digia.com/contact-us.
**
** GNU Lesser General Public License Usage
** Alternatively, this file may be used under the terms of the GNU Lesser
** General Public License version 2.1 as published by the Free Software
** Foundation and appearing in the file LICENSE.LGPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU Lesser General Public License version 2.1 requirements
** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
**
** In addition, as a special exception, Digia gives you certain additional
** rights.  These rights are described in the Digia Qt LGPL Exception
** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
**
** GNU General Public License Usage
** Alternatively, this file may be used under the terms of the GNU
** General Public License version 3.0 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.  Please review the following information to
** ensure the GNU General Public License version 3.0 requirements will be
** met: http://www.gnu.org/copyleft/gpl.html.
**
**
** $QT_END_LICENSE$
**
**************************************************************************/

#ifndef REMOTECLIENT_H
#define REMOTECLIENT_H

#include "installer_global.h"

#include <QObject>

QT_BEGIN_NAMESPACE
class QHostAddress;
class QTcpSocket;
QT_END_NAMESPACE

namespace QInstaller {

class RemoteClientPrivate;

class INSTALLER_EXPORT RemoteClient : public QObject
{
    Q_OBJECT
    Q_DISABLE_COPY(RemoteClient)
    Q_DECLARE_PRIVATE(RemoteClient)
    friend class RemoteClientGuard;

public:
    enum Mode {
        Debug,
        Release
    };

    enum StartAs {
        User,
        Administrator
    };
    static RemoteClient &instance();

    bool connect(QTcpSocket *socket) const;
    void init(quint16 port, const QHostAddress &address, Mode mode);

    QString authorizationKey() const;
    void setAuthorizationKey(const QString &key);

    bool isActive() const;
    void setActive(bool active);

    void setStartServerCommand(const QString &command, StartAs startAs);
    void setStartServerCommand(const QString &command, const QStringList &arguments, StartAs start);

private:
    RemoteClient();
    ~RemoteClient();

private:
    QScopedPointer<RemoteClientPrivate> d_ptr;
};

} // namespace QInstaller

#endif // REMOTECLIENT_H
